package net.sf.mgp;

/*
 * Copyright 2008 MGP
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.util.Iterator;

import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.StringUtils;
import org.codehaus.plexus.util.cli.CommandLineException;
import org.codehaus.plexus.util.cli.CommandLineUtils;
import org.codehaus.plexus.util.cli.Commandline;
import org.codehaus.plexus.util.cli.StreamConsumer;

/**
 * GWTCompiler goal
 * 
 * @goal compile
 * @phase process-sources
 * @requiresDependencyResolution compile
 * @description GWTCompiler goal
 * @author Maxim Gordienko
 */
public class CompileMojo extends AbstractMojo {
	private static final String GWTCOMPILER_CLASS_NAME = "com.google.gwt.dev.GWTCompiler";

	/**
	 * @parameter expression="${project}"
	 * @required
	 * @readonly
	 */
	private MavenProject project;

	/**
	 * Output directory for generated files
	 * 
	 * @parameter expression="${project.build.directory}/gwt"
	 * @required
	 */
	private File outputDirectory;

	/**
	 * Compiler JVM arguments
	 * 
	 * @parameter default-value="-Djava.awt.headless=true"
	 */
	private String jvmArgs;

	/**
	 * GWT project module
	 * 
	 * @parameter
	 * @required
	 */
	private String module;

	/**
	 * JavaScript output style OBF[USCATED], PRETTY, or DETAILED
	 * 
	 * @parameter default-value="OBF"
	 */
	private String style;

	public void execute() throws MojoExecutionException {
		if (getLog().isDebugEnabled()) {
			getLog().debug("CompileMojo#execute()");
		}

		Commandline cmd = new Commandline();
		cmd.setExecutable("java");
		if (!StringUtils.isEmpty(jvmArgs)) {
			cmd.createArg().setValue(jvmArgs);
		}
		cmd.createArg().setValue("-cp");
		cmd.createArg().setValue(getClassPath());

		cmd.createArg().setValue(GWTCOMPILER_CLASS_NAME);

		cmd.createArg().setValue("-out");
		cmd.createArg().setValue(outputDirectory.getAbsolutePath());
		cmd.createArg().setValue("-style");
		cmd.createArg().setValue(style);
		cmd.createArg().setValue(module);

		if (getLog().isDebugEnabled()) {
			getLog().debug("Invoking compiler " + cmd.toString());
		}
		StreamConsumer logStreamConsumer = new StreamConsumer() {
			public void consumeLine(String line) {
				if (getLog().isDebugEnabled()) {
					getLog().debug("Compiler output: " + line);
				}
			}
		};

		try {
			int rc = CommandLineUtils.executeCommandLine(cmd, logStreamConsumer, logStreamConsumer);
			if (rc != 0) {
				throw new MojoExecutionException("GWT compiler exited with error: " + rc);
			}
		} catch (CommandLineException e) {
			throw new MojoExecutionException("Execution failed", e);
		}
		if (getLog().isDebugEnabled()) {
			getLog().debug("CompileMojo#execute() finished");
		}
	}

	private String getClassPath() {
		String sep = System.getProperty("path.separator");
		StringBuffer sb = new StringBuffer();

		for (Iterator it = project.getCompileSourceRoots().iterator(); it.hasNext();) {
			String path = (String) it.next();
			sb.append(path).append(sep);
		}

		for (Iterator it = project.getResources().iterator(); it.hasNext();) {
			Resource resource = (Resource) it.next();
			sb.append(resource.getDirectory()).append(sep);
		}

		URLClassLoader currentClassLoader = (URLClassLoader) getClass().getClassLoader();
		URL[] originalUrls = currentClassLoader.getURLs();
		for (int i = 0; i < originalUrls.length; i++) {
			String url = originalUrls[i].toExternalForm();
			url = url.substring("file:".length());
			url = URLDecoder.decode(url);
			File file = new File(url);
			sb.append(file.getAbsolutePath()).append(sep);
		}

		return sb.toString();
	}
}
